#define _FILE_OFFSET_BITS 64

#define APP_VERSION "0.2"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* patch structure */
struct patch
{
   unsigned long offset; /* Offset */
   char values[17]; /* Values to patch */
   int nbytes; /* Bytes to patch */
};

/* NTSC to PAL patch */
struct patch ntsc2pal[]= {
                           {0x04E003, "\x02", 0x01}, /* Patch #1 */
                           {0x04E010, "\x00\x80\x80\x06\x03\x03\x04\x03\x07\x00\x00\x00\x00\x00\x00\x00", 0x10} /* Patch #2 */
                         };

/* PAL to NTSC patch */
struct patch pal2ntscu[] = {
                             {0x04E003, "\x01", 0x01}, /* Patch #1 */
                             {0x04E010, "\x80\x0D\x80\x80\x80\x80\x80\x80\x80\x80\x80\x80\x080\x80\x80\x80", 0x10} /* Patch #2 */
                           };

/* NTSC-J to NTSC-U patch */
struct patch ntscj2ntscu[] = {
                               {0x04E003, "\x01", 0x01}, /* Patch #1 */
                               {0x04E010, "\x00\x80\x80\x80\x80\x80\x80\x80\x80\x80\x80\x80\x80\x80\x80\x80", 0x10} /* Patch #2 */
                             };

/* Functions prototypes */
void pause(void);
int patchIso(char *, struct patch *, int);

int main(int argc, char *argv[])
{
	char *filename;
	int option;

	printf("----------------------------------\n");
	printf("+    SYNDICATE Region Patcher    +\n");
	printf("+ dissasembly reconstructed code +\n");
	printf("----------------------------------\n");

	printf("----------------------------------\n");
	printf("+        Region-FriiX %3s        +\n", APP_VERSION);
	printf("+     Developed by Waninkoko     +\n");
	printf("+      Based on Glicer code      +\n");
	printf("----------------------------------\n");

	/* If at least one argument has been passed, use it as filename */
	if(argc > 1)
		filename = argv[1];
	else
	{
		filename = (char *)malloc(128); /* Allocate memory */

		do
		{
			printf("\n:: ISO file to patch: ");
			scanf("%s", filename);
		} while(filename == "");
	}

	/* Show options */
	printf("\n1. NTSC-J/NTSC-U to PAL patch.");
	printf("\n2. PAL to NTSC-U patch.");
	printf("\n3. NTSC-J to NTSC-U patch.");
	printf("\n4. Quit application.\n\n");

	do
	{
		/* Get option */
		printf(":: Select an option: ");
		scanf("%d[^\n]", &option);

		fflush(stdin); /* Flush standard input */
	} while( (option < 1) || (option > 3) );

	printf("\n");

	switch(option)
	{
		/* NTSC-J/NTSC-U to PAL patch */
		case 1:
			if( patchIso(filename, ntsc2pal, 2) == 0 )
				printf("\n:: [NTSC-J/NTSC-U -> PAL] patch applied succesfully!!\n");
			else
				printf("\n:: [NTSC-J/NTSC-U -> PAL] patch failed!!\n");

			break;

		case 2:
			if( patchIso(filename, pal2ntscu, 2) == 0 )
				printf("\n:: [PAL -> NTSC-U] patch applied succesfully!!\n");
			else
				printf("\n:: [PAL -> NTSC-U] patch failed!!\n");

			break;

		/* NTSC-J to NTSC-U patch */
		case 3:
			if( patchIso(filename, ntscj2ntscu, 2) == 0 )
				printf("\n:: [NTSC-J -> NTSC-U] patch applied succesfully!!\n");
			else
				printf("\n:: [NTSC-J -> NTSC-U] patch failed!!\n");

			break;

		/* Exit */
		case 4:
			return 0;
	}

	/* Pause */
	pause();

	return 0;
}

void pause(void)
{
	printf("\n");

	fflush(stdin); /* Flush standard input */
	getchar(); /* Wait until user press a key */
}

int patchIso(char *isofile, struct patch *ppatch, int npatches)
{
	FILE *fp;
	int i;

	printf(":: Patching ISO... ");

	/* Open ISO file */
	fp = fopen(isofile, "rb+");
	if(!fp)
	{
		/* Print an error message */
		printf("ERROR!! \"%s\" not found!!", isofile);
		return 1;
	}

	/* Patch file */
	for(i = 0; i < npatches; i++)
	{
		fseek(fp, ppatch[i].offset, SEEK_SET);

		/* Check that patch is being written */
		if( fwrite(ppatch[i].values, ppatch[i].nbytes, 1, fp) < 1 )
		{
			/* Print an error message */
			printf("ERROR!! Cannot patch \"%s\". Please, check file write permissions.", isofile);

			fclose(fp);
			return 1;
		}
	}

	/* Close file */
	fclose(fp);

	printf("OK!!");

	/* Return success */
	return 0;
}
